package com.htfp.weather.griddata.operation;

import com.alicloud.openservices.tablestore.AsyncClient;
import com.alicloud.openservices.tablestore.model.*;
import com.aliyun.tablestore.grid.GridDataDeleter;
import com.aliyun.tablestore.grid.core.RequestBuilder;
import com.aliyun.tablestore.grid.model.DeleteDataParam;
import com.aliyun.tablestore.grid.model.GridDataSetMeta;
import com.htfp.weather.griddata.common.TableConfig;

import lombok.extern.slf4j.Slf4j;

import java.lang.reflect.Field;
import java.util.Set;
import java.util.concurrent.ExecutionException;
import java.util.stream.Collectors;

/**
 * @Author : shiyi
 * @Date : 2024/2/2 16:53
 * @Description : 批量删除数据
 */
@Slf4j
public class DataDeleter extends BaseTableOperation {

    /** 删除指定datasetId的所有数据
     *
     * @param dataSetId

     * @return
     * @throws Exception
     */
    public void deleteDataset(String dataSetId) throws Exception {
        GridDataDeleter deleter = tableStoreGrid.getDataDeleter(tableStoreGrid.getDataSetMeta(dataSetId));
        deleter.delete(dataSetId);
    }

    private AsyncClient getClient() throws IllegalAccessException, NoSuchFieldException {
        Field declaredField = this.tableStoreGrid.getClass().getDeclaredField("asyncClient");
        declaredField.setAccessible(true);
        AsyncClient client = (AsyncClient) declaredField.get(this.tableStoreGrid);
        return client;
    }
    private void batchWriteRow() throws Exception {
        AsyncClient client = getClient();
        BatchWriteRowRequest batchWriteRowRequest = new BatchWriteRowRequest();

        //构造rowDeleteChange1。
        // PrimaryKeyBuilder pk1Builder = PrimaryKeyBuilder.createPrimaryKeyBuilder();
        // pk1Builder.addPrimaryKeyColumn("_id", PrimaryKeyValue.fromString("pk"));
        // //设置数据表名称。
        // RowDeleteChange rowDeleteChange1 = new RowDeleteChange("<TABLE_NAME1>", pk1Builder.build());
        // //添加到batch操作中。
        // batchWriteRowRequest.addRowChange(rowDeleteChange1);
        //构造rowDeleteChange2。
        String dataSetId = "UTC-20230910";
        GridDataSetMeta dataSetMeta = this.tableStoreGrid.getDataSetMeta(dataSetId);
        Set<Integer> refTimes = dataSetMeta.getForecastHours().stream().map(Integer::parseInt).collect(Collectors.toSet());
        for (int t=0;t<3;t++) {
            for (int z=0; z<8; z++) {
                DeleteDataParam param = new DeleteDataParam(TableConfig.DATA_TABLE_NAME, "UTC-20230910", "Total_cloud_cover_isobaric", t, z);
                //添加到batch操作中。
                batchWriteRowRequest.addRowChange(RequestBuilder.buildDeleteDataRequest(param).getRowChange());
            }
        }



        BatchWriteRowResponse response = client.batchWriteRow(batchWriteRowRequest, null).get();

        System.out.println("是否全部成功：" + response.isAllSucceed());
        if (!response.isAllSucceed()) {
            for (BatchWriteRowResponse.RowResult rowResult : response.getFailedRows()) {
                System.out.println("失败的行：" + batchWriteRowRequest.getRowChange(rowResult.getTableName(), rowResult.getIndex()).getPrimaryKey());
                System.out.println("失败原因：" + rowResult.getError());
            }
            /**
             * 可以通过createRequestForRetry方法再构造一个请求对失败的行进行重试。此处只给出构造重试请求的部分。
             * 推荐的重试方法是使用SDK的自定义重试策略功能，支持对batch操作的部分行错误进行重试。设置重试策略后，调用接口处无需增加重试代码。
             */
            BatchWriteRowRequest retryRequest = batchWriteRowRequest.createRequestForRetry(response.getFailedRows());
        }
    }

    private void getRange(String pk,  String startPkValue, String endPkValue) throws NoSuchFieldException, IllegalAccessException, ExecutionException, InterruptedException {
        AsyncClient client = getClient();

        //设置数据表名称。
        RangeRowQueryCriteria rangeRowQueryCriteria = new RangeRowQueryCriteria(TableConfig.DATA_TABLE_NAME);

        //设置起始主键。
        PrimaryKeyBuilder primaryKeyBuilder = PrimaryKeyBuilder.createPrimaryKeyBuilder();
        primaryKeyBuilder.addPrimaryKeyColumn(pk, PrimaryKeyValue.fromString(startPkValue));
        rangeRowQueryCriteria.setInclusiveStartPrimaryKey(primaryKeyBuilder.build());

        //设置结束主键。
        primaryKeyBuilder = PrimaryKeyBuilder.createPrimaryKeyBuilder();
        primaryKeyBuilder.addPrimaryKeyColumn(pk, PrimaryKeyValue.fromString(endPkValue));
        rangeRowQueryCriteria.setExclusiveEndPrimaryKey(primaryKeyBuilder.build());

        rangeRowQueryCriteria.setMaxVersions(1);

        System.out.println("GetRange的结果为：");
        while (true) {
            GetRangeResponse getRangeResponse = client.getRange(new GetRangeRequest(rangeRowQueryCriteria),null).get();
            for (Row row : getRangeResponse.getRows()) {
                System.out.println(row);
            }

            //如果NextStartPrimaryKey不为null，则继续读取。
            if (getRangeResponse.getNextStartPrimaryKey() != null) {
                rangeRowQueryCriteria.setInclusiveStartPrimaryKey(getRangeResponse.getNextStartPrimaryKey());
            } else {
                break;
            }
        }
    }

    public static void main(String[] args) throws Exception {
        DataDeleter dataDeleter = new DataDeleter();
        // dataDeleter.batchWriteRow();
        try {
            dataDeleter.deleteDataset("UTC-20230910");
        } finally {
            dataDeleter.close();
        }

        // dataDeleter.getRange("_id", "UTC-20230910", "UTC-20230911");
    }
}
