package com.htfp.weather.schedule;

import com.htfp.weather.download.FileInfo;
import com.htfp.weather.download.GfsDataConfig;
import com.htfp.weather.download.GfsDownloader;
import com.htfp.weather.griddata.operation.GfsDataImport;
import com.htfp.weather.info.Constant;
import com.htfp.weather.web.exception.AppExcpetion;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.checkerframework.checker.units.qual.A;
import org.springframework.context.annotation.DependsOn;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.time.LocalDateTime;
import java.time.OffsetDateTime;
import java.time.ZoneOffset;
import java.time.format.DateTimeFormatter;
import java.util.List;

/**
 * @Author : shiyi
 * @Date : 2024/5/8 11:36
 * @Description : 更新数据库数据，包括下载、导入、清楚本地过期数据
 */
@Component
@Slf4j
@DependsOn({"gfsDataImport", "gfsDownloader"})
public class GridDataProcesser {
    @Resource
    GfsDataImport gfsDataImport;
    @Resource
    GfsDownloader gfsDownloader;
    @Resource
    GfsDataConfig gfsDataConfig;

    public void dailyDataProcess() throws Exception {
        gfsDownloader.iniTimeSetting();
        OffsetDateTime refTime = gfsDownloader.getRefTime();
        List<FileInfo> fileInfoList = gfsDownloader.getFilesInfo();
        // if (allComplete) {
            try {
                gfsDataImport.importData(refTime);
            } catch (AppExcpetion e) {
                log.info("导入数据失败");
            // }
        }
    }

    public void clearExpiredData() {
        // String dataFolder = refTime.format(DateTimeFormatter.ofPattern(Constant.DATA_FOLDER_STRING));
        File dataDir = new File(gfsDataConfig.getSaveRoot());
        if (!dataDir.exists()) {
            log.warn("文件夹 {} 不存在", dataDir);
            return;
        }
        OffsetDateTime now = OffsetDateTime.now();
        for (File subDir : dataDir.listFiles()) {
            OffsetDateTime refTime = LocalDateTime.parse(subDir.getName(), DateTimeFormatter.ofPattern(Constant.DATA_FOLDER_STRING)).atOffset(ZoneOffset.UTC);
            if (refTime.isBefore(now.minusDays(3))) {
                try {
                   FileUtils.deleteDirectory(subDir);
                    log.info("删除过期数据文件夹成功： {}", subDir.getName());
                } catch (IOException e) {
                    log.info("删除过期数据文件夹失败： {}", subDir.getName());
                }
            }
        }
    }

}
