package com.htfp.weather.utils;

import ai.djl.ndarray.NDArray;
import ai.djl.ndarray.NDManager;
import ai.djl.ndarray.types.DataType;
import ai.djl.ndarray.types.Shape;
import org.checkerframework.checker.units.qual.A;
import ucar.ma2.Array;
import ucar.ma2.ArrayFloat;
import ucar.ma2.Index;
import ucar.nc2.units.DateType;

import java.nio.ByteBuffer;
import java.text.DecimalFormat;
import java.util.Arrays;

/**
 * @Author : shiyi
 * @Date : 2024/1/24 11:32
 * @Description : 气象数据处理工具类
 */
public class MeteoUtils {

    /**
     * 风速单位转换
     *
     * @param mps 风速 (m/s)
     * @return 风速 (km/h)
     */
    public static double mPerSecond2kmPerHour(double mps) {
        return mps * 3.6;
    }

    /**
     * @param kph 风速 (km/h)
     * @return 风速 (m/s)
     */
    public static double kmPerHour2mPerSecond(double kph) {
        return kph / 3.6;
    }

    // public static double pressure2Height(double pressure) {
    //
    // }
    //
    // public static double height2Pressure(double pressure) {
    //
    // }

    /**
     * 开尔文转摄氏度
     */
    public static Array kelvin2Celsius(Array array) {
        for (int i = 0; i < array.getSize(); i++) {
            array.setFloat(i, array.getFloat(i) - 273.15f);
        }
        return array;
    }
    public static double calculateWindSpeed(double u, double v) {
        return Math.sqrt(u * u + v * v);
    }
    //
    // @Deprecated
    // public static Array calculateWindSpeed(Array u, Array v) {
    //     // // FIXME 2024/5/13: io耗时太多，对于一般大小的矩阵不使用
    //     uvValid(u, v);
    //     try (NDManager manager = NDManager.newBaseManager()) {
    //         int[] shapeArray = u.getShape();
    //         Shape shape = new Shape(Arrays.stream(shapeArray).mapToLong(i -> (long) i).toArray());
    //         NDArray uwnd = manager.create((float[]) u.getStorage(), shape);
    //         NDArray vwnd = manager.create((float[]) v.getStorage(), shape);
    //         NDArray windSpeed = (uwnd.square().add(vwnd.square())).sqrt();
    //         ByteBuffer byteBuffer = windSpeed.toByteBuffer();
    //         return Array.factory(u.getDataType(), shapeArray, byteBuffer);
    //     } catch (Exception e) {
    //         throw e;
    //     }
    // }

    private static void uvValid(Array u, Array v) {
        if (u.getSize() != v.getSize()) {
            throw new IllegalArgumentException("u 和 v 数据量不一致");
        }
        if (u.getSize() == 0 || v.getSize() == 0) {
            throw new IllegalArgumentException("u 和 v 数据不能为空");
        }
        if (!Arrays.equals(u.getShape(), v.getShape())) {
            throw new IllegalArgumentException("u 和 v 数据形状不一致");
        }
        if (!u.getDataType().equals(v.getDataType())) {
            throw new IllegalArgumentException("u 和 v 数据类型不一致");
        }
    }

    public static Array calculateWindSpeed(Array u, Array v) {
        uvValid(u, v);
        Array windSpeed = new ArrayFloat(u.getShape());
        for (int i = 0; i < windSpeed.getSize(); i++) {
            windSpeed.setFloat(i, (float) calculateWindSpeed(u.getFloat(i), v.getFloat(i)));
        }
        return windSpeed;

    }

    public static Array calculateWindDirection(Array u, Array v) {
        uvValid(u, v);
        Array windDirection = new ArrayFloat(u.getShape());
        for (int i = 0; i < windDirection.getSize(); i++) {
            windDirection.setFloat(i, (float) calculateWindDirection(u.getFloat(i), v.getFloat(i)));
        }
        return windDirection;

    }
    public static double calculateWindDirection(double u, double v) {
        // DecimalFormat df = new DecimalFormat("#.00");

        double windDirDeg;

        if (nearToZero(u) && v > 0.) {
            windDirDeg = 180;
        } else if (nearToZero(u) && v < 0.) {
            windDirDeg = 0;
        } else if (u > 0. && nearToZero(v)) {
            windDirDeg = 270;
        } else if (u < 0. && nearToZero(v)) {
            windDirDeg = 90;
        } else if (nearToZero(u) && nearToZero(v)) {
            windDirDeg = 0;
        } else {
            windDirDeg = calculateWindDirection0(u, v);
        }
        return (float) windDirDeg;
    }

    public static double calculateWindDirection0(double u, double v) {
        // DecimalFormat df = new DecimalFormat("#.00");
        double windDirRad = Math.atan2(v, u);
        return (270 - windDirRad * 180 / Math.PI) % 360;
    }

    private static boolean nearToZero(double value) {
        return Math.abs(value) < 1e-5;
    }
}
