package com.htfp.weather.web.controller;

import com.htfp.weather.download.FileInfo;
import com.htfp.weather.download.GfsDataConfig;
import com.htfp.weather.download.GfsDownloader;
import com.htfp.weather.griddata.common.TableConfigBean;
import com.htfp.weather.griddata.operation.GfsDataImport;
import com.htfp.weather.web.exception.AppExcpetion;
import com.htfp.weather.web.exception.ErrorCode;
import com.htfp.weather.web.pojo.response.Result;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.io.IOException;
import java.time.OffsetDateTime;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Author : shiyi
 * @Date : 2024/4/21 15:56
 * @Description : 配置文件管理
 */
@CrossOrigin
@RestController
@RequestMapping("/htfp/weather/config")
public class ConfigController {
    @Resource
    TableConfigBean tableConfigBean;
    @Resource
    GfsDataConfig gfsDataConfig;
    @Resource
    GfsDownloader gfsDownloader;
    @Resource
    GfsDataImport gfsDataImport;

    private final String SECRET = "htfpweather";

    private void validSecret(String secret) {
        if (!SECRET.equals(secret)) {
            throw new AppExcpetion(ErrorCode.SECRET_ERROR);
        }
    }
    @RequestMapping("/queryDatabaseConfig")
    public Result queryDatabaseConfig() {
        return Result.success(tableConfigBean);
    }

    @RequestMapping("/updateDatabaseConfig")
    public Result updateDatabaseConfig(@RequestParam String secret, @Validated @RequestBody TableConfigBean updateConfig) {
        validSecret(secret);
        if (updateConfig == null) {
            throw new AppExcpetion(ErrorCode.CONFIG_ERROR, "配置文件不能为空");
        }
        updateConfig.valid();
        try {
            tableConfigBean.writeConfig(updateConfig);
        } catch (IOException e) {
            throw new AppExcpetion(ErrorCode.CONFIG_ERROR, e);
        }
        return Result.success();
    }

    @RequestMapping("/queryDataSourceConfig")
    public Result queryDataSourceConfig() {
        return Result.success(gfsDataConfig);
    }

    @RequestMapping("/updateDataSourceConfig")
    public Result updateDataSourceConfig(@RequestParam String secret, @Validated @RequestBody GfsDataConfig updateConfig) {
        validSecret(secret);
        if (updateConfig == null) {
            throw new AppExcpetion(ErrorCode.CONFIG_ERROR, "配置文件不能为空");
        }
        updateConfig.valid();
        try {
            gfsDataConfig.writeConfig(updateConfig);
        } catch (IOException e) {
            throw new AppExcpetion(ErrorCode.CONFIG_ERROR, e);
        }
        return Result.success();
    }

    @RequestMapping("/download")
    public Result download(@RequestParam String secret) {
        validSecret(secret);
        // List<FileInfo> allCompleted = false;
        try {
            gfsDownloader.iniTimeSetting();
            List<FileInfo> fileInfoList = gfsDownloader.getFilesInfo();
            List<FileInfo> finishedList = gfsDownloader.downloadAll(fileInfoList);
            List<FileInfo> successList = finishedList.stream().filter(FileInfo::isDownloadSuccess).collect(Collectors.toList());
            List<FileInfo> failedList = finishedList.stream().filter(fileInfo -> !fileInfo.isDownloadSuccess()).collect(Collectors.toList());
            Map<String, List<FileInfo>> data = new HashMap<>();
            data.put("successList", successList);
            data.put("failedList", failedList);
            if (CollectionUtils.isEmpty(failedList)) {
                return Result.success(data);
            } else {
                return new Result(false, ErrorCode.DOWNLOAD_ERROR.getCode(), ErrorCode.DOWNLOAD_ERROR.getMsg() + ":下载未全部完成", data);
            }
        } catch (Exception e) {
            return Result.error(ErrorCode.DOWNLOAD_START_ERROR, e.getMessage());
        }
    }


    @RequestMapping("/import")
    public Result importData(@RequestBody Map<String, String> params) {
        String secret = params.get("secret");
        validSecret(secret);
        OffsetDateTime time = OffsetDateTime.parse(params.get("time"));
        try {
            gfsDataImport.importData(time);
            return Result.success();
        } catch (Exception e) {
            e.printStackTrace();
            return Result.error(e);
        }
    }

}
