package com.htfp.weather.web.controller;

import com.htfp.weather.utils.DateTimeUtils;
import com.htfp.weather.web.pojo.request.PlaneRequest;
import com.htfp.weather.web.pojo.request.Position3D;
import com.htfp.weather.web.pojo.request.ProfileRequest;
import com.htfp.weather.web.pojo.response.*;
import com.htfp.weather.web.service.GfsDataServiceImpl;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.time.OffsetDateTime;

/**
 * @Author : shiyi
 * @Date : 2024/5/8 15:15
 * @Description :
 */
@CrossOrigin
@RestController
@RequestMapping("/htfp/weather/upper/")
public class UpperWeatherController {

    @Resource(name = "tablestore-gfs")
    GfsDataServiceImpl gfsDataService;

    @RequestMapping("/profileByPressure")
    public Result queryProfile(@Validated @RequestBody ProfileRequest profileRequest) {
        OffsetDateTime time = OffsetDateTime.parse(profileRequest.getTime());
        OffsetDateTime utcDateTime = DateTimeUtils.getUTCDateTime(time);
        String variableName = profileRequest.getVariableName();
        double latitude = profileRequest.getLatitude();
        double longitude = profileRequest.getLongitude();
        ProfileResponse profileResponse = gfsDataService.getProfile(utcDateTime, variableName, latitude, longitude);
        return Result.success(profileResponse);
    }

    @PostMapping("/forecast")
    public Result queryTimeSeries(@Validated @RequestBody Position3D position3D) {
        double latitude = position3D.getLatitude();
        double longitude = position3D.getLongitude();
        int level = position3D.getLevel();
        TimeSeriesDataset forecastSeries = gfsDataService.getForecastSeries(latitude, longitude, level);
        return Result.success(forecastSeries);
    }

    @PostMapping("/plane")
    public Result queryPlane(@Validated @RequestBody PlaneRequest planeRequest) {
        planeRequest.valid();
        String variableName = planeRequest.getVariableName();
        OffsetDateTime time = OffsetDateTime.parse(planeRequest.getTime());
        OffsetDateTime utcDateTime = DateTimeUtils.getUTCDateTime(time);
        int level = planeRequest.getLevel();
        double minLat = planeRequest.getMinLat();
        double maxLat = planeRequest.getMaxLat();
        double minLon = planeRequest.getMinLon();
        double maxLon = planeRequest.getMaxLon();
        PlaneResponse forecastSeries = gfsDataService.getPlane(utcDateTime, variableName, level, minLat, maxLat, minLon, maxLon);
        return Result.success(forecastSeries);
    }

    @PostMapping("/now")
    public Result queryNowWeather(@Validated @RequestBody Position3D position3D) {
        double latitude = position3D.getLatitude();
        double longitude = position3D.getLongitude();
        int level = position3D.getLevel();
        NowWeatherStatus nowWeather = gfsDataService.getNowWeather(latitude, longitude, level);
        return Result.success(nowWeather);
    }
}
