package com.htfp.weather.web.pojo.request;

import com.htfp.weather.griddata.common.GfsVariableIsobaricEnum;
import com.htfp.weather.info.Constant;
import com.htfp.weather.info.GfsLevelsEnum;
import com.htfp.weather.web.exception.AppExcpetion;
import com.htfp.weather.web.exception.ErrorCode;
import com.htfp.weather.web.valid.DateTimeStr;
import com.htfp.weather.web.valid.EnumValid;
import lombok.Data;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;

/**
 * @Author : shiyi
 * @Date : 2024/5/15 16:11
 * @Description : 平面数据请求体
 */
@Data
public class PlaneRequest {
    @NotNull(message = "时间不能为空")
    @DateTimeStr(format = Constant.API_TIME_STRING, message = "时间格式错误, 应为" + "yyyy-MM-ddTHH:mmxxx")
    String time;

    @NotNull(message = "变量名不能为空")
    @EnumValid(enumClass = GfsVariableIsobaricEnum.class, usedField = "getNameInApi", message = "变量不存在")
    String variableName;

    @NotNull(message = "高度层不能为空")
    @EnumValid(enumClass = GfsLevelsEnum.class, usedField = "getCode", message = "高度层不存在")
    Integer level;

    @NotNull
    @Min(value = 70, message = "经度最小值为70（东经180°）")
    @Max(value = 140, message = "经度最大值为140（东经140°）")
    private Double minLon;

    @NotNull
    @Min(value = 70, message = "经度最小值为70（东经180°）")
    @Max(value = 140, message = "经度最大值为140（东经140°）")
    private Double maxLon;

    @NotNull
    @Min(value = 0, message = "纬度最小值为0（0°）")
    @Max(value = 55, message = "纬度最大值为55（北纬55°）")
    private Double minLat;

    @NotNull
    @Min(value = 0, message = "纬度最小值为0（0°）")
    @Max(value = 55, message = "纬度最大值为55（北纬55°）")
    private Double maxLat;

    public void valid() {
        // TODO 2024/4/21: 校验
        if (minLon >= maxLon) {
            throw new AppExcpetion(ErrorCode.VALIDATE_ERROR, "最小经度必须小于最大经度");
        }
        if (minLat >= maxLat) {
            throw new AppExcpetion(ErrorCode.VALIDATE_ERROR, "最小纬度必须小于最大纬度度");
        }
    }
}
